@@ -60,8 +60,10 @@ expandFileName <- function (fileName, base = getwd())
     # Remove all instances of '/.' (which are redundant), recursively collapse
     # instances of '/..', and remove trailing slashes
     fileName <- gsub("/\\.(?=/)", "", fileName, perl=TRUE)
-    while (length(grep('/../', fileName, fixed=TRUE) > 0))
+    while (length(grep("/../", fileName, fixed=TRUE) > 0))
         fileName <- sub("/[^/]*[^./][^/]*/\\.\\.(?=/)", "", fileName, perl=TRUE)
+    if (length(grep("/..$", fileName, perl=TRUE) > 0))
+        fileName <- sub("/[^/]*[^./][^/]*/\\.\\.$", "", fileName, perl=TRUE)
     fileName <- gsub("/*\\.?$", "", fileName, perl=TRUE)
     
     return (fileName)
